@@ -39,11 +39,19 @@
 #include "braille.h"
 #include "canvas.h"
 
+/* CSI parsing status */
+enum
+  {
+    CSI_PB=0, CSI_PX=1, CSI_PY=2, CSI_DONE=3,
+    CSI_ESC_SEEN, CSI_BEGIN, CSI_INACTIVE, CSI_INVALID
+  };
+
 static int  CountChars __P((int));
 static int  DoAddChar __P((int));
 static int  BlankResize __P((int, int));
 static int  CallRewrite __P((int, int, int, int));
 static void disp_readev_fn __P((struct event *, char *));
+static void disp_processinput __P((struct display *, unsigned char *, int));
 static void disp_writeev_fn __P((struct event *, char *));
 #ifdef linux
 static void disp_writeev_eagain __P((struct event *, char *));
@@ -94,6 +102,7 @@ extern struct winsize glwz;
 #endif
 extern char **NewEnv;
 extern int real_uid, real_gid;
+extern int ServerSocket, eff_uid, eff_gid;
 #endif
 
 /*
@@ -487,6 +496,7 @@ FinitTerm()
       if (D_mousetrack)
 	D_mousetrack = 0;
       MouseMode(0);
+      ExtMouseMode(0);
       SetRendition(&mchar_null);
       SetFlow(FLOW_NOW);
 #ifdef MAPKEYS
@@ -833,6 +843,31 @@ int mode;
           AddStr(mousebuf);
 	}
       D_mouse = mode;
+      D_mouse_parse.state = CSI_INACTIVE;
+    }
+}
+
+void
+ExtMouseMode(mode)
+     int mode;
+{
+  if (display && D_extmouse != mode)
+    {
+      char mousebuf[20];
+      if (!D_CXT)
+        return;
+      if (D_extmouse)
+        {
+          sprintf(mousebuf, "\033[?%dl", D_extmouse);
+          AddStr(mousebuf);
+        }
+      if (mode)
+        {
+          sprintf(mousebuf, "\033[?%dh", mode);
+          AddStr(mousebuf);
+        }
+      D_extmouse = mode;
+      D_mouse_parse.state = CSI_INACTIVE;
     }
 }
 
@@ -1261,6 +1296,7 @@ int cur_only;
   CursorkeysMode(0);
   CursorVisibility(0);
   MouseMode(0);
+  ExtMouseMode(0);
   SetRendition(&mchar_null);
   SetFlow(FLOW_NOW);
 
@@ -2898,12 +2934,14 @@ int newtop, newbot;
 #define WT_FLAG "2"	/* change to "0" to set both title and icon */
 
 void
-SetXtermOSC(i, s)
+SetXtermOSC(i, s, t)
 int i;
 char *s;
+char *t;
 {
   static char *oscs[][2] = {
     { WT_FLAG ";", "screen" }, /* set window title */
+    { "11;", ""},       /* background RGB */
     { "20;", "" },      /* background */
     { "39;", "black" }, /* default foreground (black?) */
     { "49;", "white" }  /* default background (white?) */
@@ -2924,15 +2962,15 @@ char *s;
   AddStr("\033]");
   AddStr(oscs[i][0]);
   AddStr(s);
-  AddChar(7);
+  AddStr(t);
 }
 
 void
 ClearAllXtermOSC()
 {
   int i;
-  for (i = 3; i >= 0; i--)
-    SetXtermOSC(i, 0);
+  for (i = 4; i >= 0; i--)
+    SetXtermOSC(i, 0, "\a");
   if (D_xtermosc[0])
     AddStr("\033[23;" WT_FLAG "t");	/* unstack titles (xterm patch #251) */
 }
@@ -3159,6 +3197,7 @@ NukePending()
   int oldkeypad = D_keypad, oldcursorkeys = D_cursorkeys;
   int oldcurvis = D_curvis;
   int oldmouse = D_mouse;
+  int oldextmouse = D_extmouse;
 
   oldrend = D_rend;
   len = D_obufp - D_obuf;
@@ -3221,6 +3260,7 @@ NukePending()
   CursorkeysMode(oldcursorkeys);
   CursorVisibility(oldcurvis);
   MouseMode(oldmouse);
+  ExtMouseMode(oldextmouse);
   if (D_CWS)
     {
       debug("ResizeDisplay: using WS\n");
@@ -3349,13 +3389,24 @@ char *data;
     }
 }
 
+/* maximum mouse sequence length is SGR: ESC [ < b ; x ; y M */
+#define MAX_MOUSE_SEQUENCE (3+10+1+10+1+10+1)
+
 static void
 disp_readev_fn(ev, data)
 struct event *ev;
 char *data;
 {
+  /* We have to intercept mouse sequences in order to translate them
+   * properly, and an incoming sequence could be spread over multiple
+   * I/O reads. When this occurs, we buffer intermediate state in
+   * D_mouse_params, and then use the reservation at the front of
+   * bufspace to prepend the completed and translated mouse sequence.
+   */
   int size;
-  char buf[IOSIZE];
+  char bufspace[MAX_MOUSE_SEQUENCE + IOSIZE];
+  unsigned char *buf = bufspace + MAX_MOUSE_SEQUENCE;
+
   struct canvas *cv;
 
   display = (struct display *)data;
@@ -3442,63 +3493,247 @@ char *data;
     ResetIdle();
   if (D_fore)
     D_fore->w_lastdisp = display;
+
   if (D_mouse && D_forecv)
     {
       unsigned char *bp = (unsigned char *)buf;
-      int x, y, i;
+      unsigned char *end = bp + size;
+      unsigned char *mark = NULL;
 
-      /* XXX this assumes that the string is read in as a whole... */
-      for (i = size; i > 0; i--, bp++)
+      /* When mouse mode is enabled, buffer up incoming CSI until we
+       * know whether it is a mouse sequence. If not a mouse event,
+       * emit the CSI unchanged; if an invalid mouse event, swallow
+       * it; otherwise, translate the sequence and emit it.
+       *
+       * Supported mouse events take two flavors.
+       *
+       * VT200: CSI M Cb Cx Cy
+       * SGR:   CSI < Ps ; Ps ; Ps M|m
+       *
+       * UTF-8 and UXRVT modes are explicitly rejected because they
+       * introduce incompatibilities with other control sequences.
+       *
+       * NOTE: applications wishing to use SGR mode will normally
+       * enable VT200 mode first as a fallback in case the terminal
+       * does not support SGR mode. Thus, we must expect to receive
+       * either mode's sequences when mouse mode is enabled. We will
+       * dutifully translate whatever arrives, without attempting to
+       * convert between modes on behalf of the application.
+       */
+      switch (D_mouse_parse.state)
+        {
+        case CSI_PY:
+        case CSI_PX:
+        case CSI_PB:
+          /* Partial mouse sequence; do not restore suppressed
+           * characters. We will emit a translated version (if valid)
+           * or swallow them (otherwise).
+           */
+          break;
+
+        case CSI_BEGIN:
+          /* Partial CSI; restore suppressed characters in case it
+           * turns out not to be a mouse sequence after all.
+           */
+          *(--buf) = '[';
+          ++size;
+          /* fall through */
+
+        case CSI_ESC_SEEN:
+          /* Escape character; restore it in case this turns out not
+           * to be the start of a mouse sequence after all.
+           */
+          *(--buf) = '\033';
+          ++size;
+          break;
+
+        default:
+          break;
+        };
+
+      while (bp != end)
 	{
-	  if (i > 5 && bp[0] == 033 && bp[1] == '[' && bp[2] == 'M')
+	  unsigned char c = *(bp++);
+
+	  switch (D_mouse_parse.state)
 	    {
-	      bp++;
-	      i--;
-	    }
-	  else if (i < 5 || bp[0] != 0233 || bp[1] != 'M')
-	    continue;
-	  x = bp[3] - 33;
-	  y = bp[4] - 33;
-	  if (x >= D_forecv->c_xs && x <= D_forecv->c_xe && y >= D_forecv->c_ys && y <= D_forecv->c_ye)
-	    {
-	      if ((D_fore && D_fore->w_mouse) || (D_mousetrack && D_forecv->c_layer->l_mode == 1))
-		{
-		  /* Send clicks only if the window is expecting clicks */
-		  x -= D_forecv->c_xoff;
-		  y -= D_forecv->c_yoff;
-		  if (x >= 0 && x < D_forecv->c_layer->l_width && y >= 0 && y < D_forecv->c_layer->l_height)
+	    case CSI_INACTIVE:
+              if (c == '\033')
+                {
+                  /* potential escape sequence */
+                  mark = bp-1;
+                  D_mouse_parse.state = CSI_ESC_SEEN;
+                }
+	      break;
+
+	    case CSI_ESC_SEEN:
+              if (c == '[')
+                {
+                  /* continue buffering an escape sequence */
+                  D_mouse_parse.state = CSI_BEGIN;
+                }
+              else
+                  D_mouse_parse.state = CSI_INACTIVE;
+	      break;
+
+	    case CSI_BEGIN:
+              if (c == 'M')
+                {
+                  /* VT200 mouse sequence */
+                  D_mouse_parse.state = CSI_PB;
+                  D_mouse_parse.sgrmode = 0;
+                }
+              else if (c == '<')
+                {
+                  /* SGR mouse sequence */
+                  D_mouse_parse.state = CSI_PB;
+                  D_mouse_parse.params[D_mouse_parse.state] = 0;
+                  D_mouse_parse.sgrmode = 1;
+                }
+              else
+                  D_mouse_parse.state = CSI_INACTIVE;
+	      break;
+
+	    case CSI_PB:
+	    case CSI_PX:
+	    case CSI_PY:
+              if (D_mouse_parse.sgrmode)
+                {
+                  /* SGR mode: parse decimal numbers */
+		  if ('0' <= c && c <= '9')
 		    {
-		      bp[3] = x + 33;
-		      bp[4] = y + 33;
-		      i -= 4;
-		      bp += 4;
-		      continue;
-		    }
-		}
+                      D_mouse_parse.params[D_mouse_parse.state] *= 10;
+                      D_mouse_parse.params[D_mouse_parse.state] += c - '0';
+                    }
+                  else if (D_mouse_parse.state == CSI_PY)
+                    {
+                      if (c == 'M' || c == 'm')
+                        D_mouse_parse.state = CSI_DONE;
+                      else
+                        D_mouse_parse.state = CSI_INVALID;
+                    }
+                  else if (c == ';')
+                    {
+                      D_mouse_parse.state++;
+                      D_mouse_parse.params[D_mouse_parse.state] = 0;
+                    }
+                  else
+                    D_mouse_parse.state = CSI_INVALID;
+                }
+              else
+                {
+                  /* VT200 mode: read raw binary values */
+                  D_mouse_parse.params[D_mouse_parse.state++] = c;
+                }
+              break;
+
+	    default:
+	      break;
 	    }
-	  else if (D_mousetrack && bp[2] == '#')
+
+          if (D_mouse_parse.state == CSI_INVALID)
+            {
+              /* swallow invalid sequence, but emit whatever came before */
+              if (buf < mark)
+                disp_processinput(display, buf, mark-buf);
+
+              buf = bp;
+              size = end - bp;
+              D_mouse_parse.state = CSI_INACTIVE;
+            }
+	  else if (D_mouse_parse.state == CSI_DONE)
 	    {
-	      /* 'focus' to the clicked region, only on mouse up */
-	      struct canvas *cv = FindCanvas(x, y);
-	      if (cv)
-		{
-		  SetForeCanvas(display, cv);
-		  /* XXX: Do we want to reset the input buffer? */
-		}
-	    }
-	  if (bp[0] == '[')
-	    {
-	      bcopy((char *)bp + 1, (char *)bp, i);
-	      bp--;
-	      size--;
-	    }
-	  if (i > 5)
-	    bcopy((char *)bp + 5, (char *)bp, i - 5);
-	  bp--;
-	  i -= 4;
-	  size -= 5;
+              /* emit whatever came before this sequence */
+              if (buf < mark)
+                disp_processinput(display, buf, mark-buf);
+
+              buf = bp;
+              size = end - bp;
+
+	      int x = D_mouse_parse.params[CSI_PX];
+              int y = D_mouse_parse.params[CSI_PY];
+              int bias = D_mouse_parse.sgrmode? 1 : 33;
+
+              x -= bias;
+              y -= bias;
+
+	      if (x >= D_forecv->c_xs && x <= D_forecv->c_xe && y >= D_forecv->c_ys && y <= D_forecv->c_ye)
+	        {
+	          if ((D_fore && D_fore->w_mouse) || (D_mousetrack && D_forecv->c_layer->l_mode == 1))
+	            {
+	              /* Send clicks only if the window is expecting clicks */
+	              x -= D_forecv->c_xoff;
+	              y -= D_forecv->c_yoff;
+
+	              if (x >= 0 && x < D_forecv->c_layer->l_width && y >= 0 && y < D_forecv->c_layer->l_height)
+	                {
+                          char tmp[MAX_MOUSE_SEQUENCE+1];
+                          int n;
+
+                          x += bias;
+                          y += bias;
+
+                          if (D_mouse_parse.sgrmode)
+                            {
+                              n = snprintf(
+                                      tmp, MAX_MOUSE_SEQUENCE, "\033[<%d;%d;%d%c",
+                                      D_mouse_parse.params[CSI_PB], x, y, c);
+                            }
+                          else
+                            {
+                              n = snprintf(
+                                      tmp, MAX_MOUSE_SEQUENCE, "\033[M%c%c%c",
+                                      D_mouse_parse.params[CSI_PB], x, y);
+                            }
+
+                          if (n > MAX_MOUSE_SEQUENCE)
+                                n = MAX_MOUSE_SEQUENCE;
+
+                          /* emit sequence */
+                          buf -= n;
+                          size += n;
+                          memcpy(buf, tmp, n);
+                        }
+                    }
+                }
+	      else if (D_mousetrack)
+                {
+	          /* 'focus' to the clicked region, only on mouse up */
+                  int focus = 0;
+                  if (D_mouse_parse.sgrmode)
+                    focus = (c == 'm');
+                  else
+                    focus = (D_mouse_parse.params[CSI_PB] == '#');
+
+	          struct canvas *cv = FindCanvas(x, y);
+	          if (focus && cv)
+	            {
+	              SetForeCanvas(display, cv);
+	              /* XXX: Do we want to reset the input buffer? */
+	            }
+	        }
+
+	      D_mouse_parse.state = CSI_INACTIVE;
+            }
 	}
+
+      if (D_mouse_parse.state != CSI_INACTIVE)
+        {
+          /* suppress partial sequence at end */
+          size = mark? mark - buf : 0;
+        }
     }
+
+  if (size > 0)
+    disp_processinput(display, buf, size);
+}
+
+static void
+disp_processinput(display, buf, size)
+     struct display* display;
+     unsigned char* buf;
+     int size;
+{
 #ifdef ENCODINGS
   if (D_encoding != (D_forecv ? D_forecv->c_layer->l_encoding : 0))
     {
@@ -3724,6 +3959,7 @@ char **cmdv;
   char *m;
   int pid;
   int slave = -1;
+  int ptype = 0;
   char termname[MAXTERMLEN + 6];
 #ifndef TIOCSWINSZ
   char libuf[20], cobuf[20];
@@ -3735,9 +3971,9 @@ char **cmdv;
   termname[sizeof(termname) - 1] = 0;
   KillBlanker();
   D_blankerpid = -1;
-  if ((D_blankerev.fd = OpenPTY(&m)) == -1)
+  if ((D_blankerev.fd = OpenDevice(cmdv, 0, &ptype, &m)) == -1)
     {
-      Msg(0, "OpenPty failed");
+      Msg(0, "OpenDevice failed");
       return;
     }
 #ifdef O_NOCTTY
@@ -3762,25 +3998,51 @@ char **cmdv;
       return;
     case 0:
       displays = 0;
-#ifdef DEBUG
-      if (dfp && dfp != stderr)
-	{
-	  fclose(dfp);
-	  dfp = 0;
-	}
+      ServerSocket = -1;
+#ifdef SIGPIPE
+      signal(SIGPIPE, SIG_DFL);
 #endif
       if (setgid(real_gid) || setuid(real_uid))
         Panic(errno, "setuid/setgid");
+      eff_uid = real_uid;
+      eff_gid = real_gid;
       brktty(D_userfd);
       freetty();
+#ifdef DEBUG
+      if (dfp && dfp != stderr)
+	  fclose(dfp);
+#endif
+      if (slave != -1)
+	{
+	  close(0);
+	  dup(slave);
+	  close(slave);
+	  closeallfiles(D_blankerev.fd);
+	  slave = dup(0);
+	}
+      else
+        closeallfiles(D_blankerev.fd);
+#ifdef DEBUG
+      if (dfp)
+        {
+          char buf[256];
+
+	  sprintf(buf, "%s/screen.blanker", DEBUGDIR);
+	  if ((dfp = fopen(buf, "a")) == 0)
+	    dfp = stderr;
+	  else
+	    (void) chmod(buf, 0666);
+	}
+      debug1("=== RunBlanker: pid %d\n", (int)getpid());
+#endif
       close(0);
       close(1);
       close(2);
-      closeallfiles(slave);
       if (open(m, O_RDWR))
 	Panic(errno, "Cannot open %s", m);
       dup(0);
       dup(0);
+      close(D_blankerev.fd);
       if (slave != -1)
 	close(slave);
       InitPTY(0);
@@ -3794,17 +4056,17 @@ char **cmdv;
       glwz.ws_row = D_height;
       (void)ioctl(0, TIOCSWINSZ, (char *)&glwz);
 #else
+      /* Always turn off nonblocking mode */
+      (void)fcntl(0, F_SETFL, 0);
       sprintf(libuf, "LINES=%d", D_height);
       sprintf(cobuf, "COLUMNS=%d", D_width);
       *np++ = libuf;
       *np++ = cobuf;
 #endif
-#ifdef SIGPIPE
-      signal(SIGPIPE, SIG_DFL);
-#endif
-      display = 0;
+      debug1("calling execvpe %s\n", *cmdv);
       execvpe(*cmdv, cmdv, NewEnv + 3);
-      Panic(errno, "%s", *cmdv);
+      debug1("exec error: %d\n", errno);
+      Panic(errno, "Cannot exec '%s'", *cmdv);
     default:
       break;
     }
