@@ -1190,14 +1190,13 @@ int c;
     {0x3000, 0x303E},
     {0x3041, 0x3096},
     {0x3099, 0x30FF},
-    {0x3105, 0x312D},
+    {0x3105, 0x312F},
     {0x3131, 0x318E},
     {0x3190, 0x31BA},
     {0x31C0, 0x31E3},
     {0x31F0, 0x321E},
     {0x3220, 0x3247},
-    {0x3250, 0x32FE},
-    {0x3300, 0x4DBF},
+    {0x3250, 0x4DBF},
     {0x4E00, 0xA48C},
     {0xA490, 0xA4C6},
     {0xA960, 0xA97C},
@@ -1209,10 +1208,13 @@ int c;
     {0xFE68, 0xFE6B},
     {0xFF01, 0xFF60},
     {0xFFE0, 0xFFE6},
-    {0x16FE0, 0x16FE0},
-    {0x17000, 0x187EC},
+    {0x16FE0, 0x16FE3},
+    {0x17000, 0x187F7},
     {0x18800, 0x18AF2},
-    {0x1B000, 0x1B001},
+    {0x1B000, 0x1B11E},
+    {0x1B150, 0x1B152},
+    {0x1B164, 0x1B167},
+    {0x1B170, 0x1B2FB},
     {0x1F004, 0x1F004},
     {0x1F0CF, 0x1F0CF},
     {0x1F18E, 0x1F18E},
@@ -1221,6 +1223,7 @@ int c;
     {0x1F210, 0x1F23B},
     {0x1F240, 0x1F248},
     {0x1F250, 0x1F251},
+    {0x1F260, 0x1F265},
     {0x1F300, 0x1F320},
     {0x1F32D, 0x1F335},
     {0x1F337, 0x1F37C},
@@ -1242,16 +1245,20 @@ int c;
     {0x1F680, 0x1F6C5},
     {0x1F6CC, 0x1F6CC},
     {0x1F6D0, 0x1F6D2},
+    {0x1F6D5, 0x1F6D5},
     {0x1F6EB, 0x1F6EC},
-    {0x1F6F4, 0x1F6F6},
-    {0x1F910, 0x1F91E},
-    {0x1F920, 0x1F927},
-    {0x1F930, 0x1F930},
-    {0x1F933, 0x1F93E},
-    {0x1F940, 0x1F94B},
-    {0x1F950, 0x1F95E},
-    {0x1F980, 0x1F991},
-    {0x1F9C0, 0x1F9C0},
+    {0x1F6F4, 0x1F6FA},
+    {0x1F7E0, 0x1F7EB},
+    {0x1F90D, 0x1F971},
+    {0x1F973, 0x1F976},
+    {0x1F97A, 0x1F9A2},
+    {0x1F9A5, 0x1F9AA},
+    {0x1F9AE, 0x1F9CA},
+    {0x1F9CD, 0x1F9FF},
+    {0x1FA70, 0x1FA73},
+    {0x1FA78, 0x1FA7A},
+    {0x1FA80, 0x1FA82},
+    {0x1FA90, 0x1FA95},
     {0x20000, 0x2FFFD},
     {0x30000, 0x3FFFD},
   };
