@@ -904,6 +904,23 @@ VA_DECL
   }
 }
 
+void
+LExtMouseMode(l, on)
+struct layer *l;
+int on;
+{
+  struct canvas *cv;
+  for (cv = l->l_cvlist; cv; cv = cv->c_lnext)
+    {
+      display = cv->c_display;
+      if (D_blocked)
+	continue;
+      if (cv != D_forecv)
+	continue;
+      ExtMouseMode(on);
+    }
+}
+
 
 /*******************************************************************/
 /*******************************************************************/
